@@ -299,6 +299,37 @@ def _detect_vcs(expected_vcs: Optional[Vcs] = None) -> Vcs:
         raise RuntimeError("Unable to detect version control system.")
 
 
+def _detect_vcs_from_archival() -> Optional[Vcs]:
+    archival = _find_higher_file(".git_archival.json", ".git")
+    if archival is not None:
+        content = archival.read_text("utf8")
+        if "$Format:" not in content:
+            return Vcs.Git
+
+    archival = _find_higher_file(".hg_archival.txt", ".hg")
+    if archival is not None:
+        return Vcs.Mercurial
+
+    return None
+
+
+def _find_higher_file(name: str, limit: Optional[str] = None, start: Path = None) -> Optional[Path]:
+    """
+    :param name: Bare name of a file we'd like to find.
+    :param limit: Give up if we find a file/folder with this name.
+    :param start: Begin recursing from this folder (default: `.`).
+    """
+
+    if start is None:
+        start = Path.cwd()
+    for level in [start, *start.parents]:
+        if (level / name).is_file():
+            return level / name
+        if limit is not None and (level / limit).exists():
+            return None
+    return None
+
+
 class _GitRefInfo:
     def __init__(
         self, ref: str, commit: str, creatordate: str, committerdate: str, taggerdate: str
@@ -407,7 +438,9 @@ class Version:
         tagged_metadata: Optional[str] = None,
         epoch: Optional[int] = None,
         branch: Optional[str] = None,
+        # fmt: off
         timestamp: Optional[dt.datetime] = None
+        # fmt: on
     ) -> None:
         """
         :param base: Release segment, such as 0.1.0.
@@ -777,6 +810,36 @@ class Version:
                 bumped.revision = bumped.revision + 1
         return bumped
 
+    @classmethod
+    def _fallback(
+        cls,
+        strict: bool,
+        *,
+        stage: Optional[Tuple[str, Optional[int]]] = None,
+        distance: int = 0,
+        commit: Optional[str] = None,
+        dirty: Optional[bool] = None,
+        tagged_metadata: Optional[str] = None,
+        epoch: Optional[int] = None,
+        branch: Optional[str] = None,
+        # fmt: off
+        timestamp: Optional[dt.datetime] = None
+        # fmt: on
+    ):
+        if strict:
+            raise RuntimeError("No tags available and fallbacks disabled by strict mode")
+        return cls(
+            "0.0.0",
+            stage=stage,
+            distance=distance,
+            commit=commit,
+            dirty=dirty,
+            tagged_metadata=tagged_metadata,
+            epoch=epoch,
+            branch=branch,
+            timestamp=timestamp,
+        )
+
     @classmethod
     def from_git(
         cls,
@@ -784,6 +847,7 @@ class Version:
         latest_tag: bool = False,
         tag_branch: Optional[str] = None,
         full_commit: bool = False,
+        strict: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on Git tags.
@@ -796,7 +860,74 @@ class Version:
         :param tag_branch: Branch on which to find tags, if different than the
             current branch.
         :param full_commit: Get the full commit hash instead of the short form.
+        :param strict: When there are no tags, fail instead of falling back to 0.0.0.
         """
+        archival = _find_higher_file(".git_archival.json", ".git")
+        if archival is not None:
+            content = archival.read_text("utf8")
+            if "$Format:" not in content:
+                data = json.loads(content)
+
+                if full_commit:
+                    commit = data.get("hash-full")
+                else:
+                    commit = data.get("hash-short")
+
+                timestamp = None
+                raw_timestamp = data.get("timestamp")
+                if raw_timestamp:
+                    timestamp = _parse_git_timestamp_iso_strict(raw_timestamp)
+
+                branch = None
+                refs = data.get("refs")
+                if refs:
+                    parts = refs.split(" -> ")
+                    if len(parts) == 2:
+                        branch = parts[1].split(", ")[0]
+
+                tag = None
+                distance = 0
+                dirty = None
+                describe = data.get("describe")
+                if describe:
+                    if describe.endswith("-dirty"):
+                        dirty = True
+                        describe = describe[:-6]
+                    else:
+                        dirty = False
+                    parts = describe.rsplit("-", 2)
+                    tag = parts[0]
+                    if len(parts) > 1:
+                        distance = int(parts[1])
+
+                if tag is None:
+                    return cls._fallback(
+                        strict,
+                        distance=distance,
+                        commit=commit,
+                        dirty=dirty,
+                        branch=branch,
+                        timestamp=timestamp,
+                    )
+
+                tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
+                    pattern, [tag], latest_tag
+                )
+                version = cls(
+                    base,
+                    stage=stage,
+                    distance=distance,
+                    commit=commit,
+                    dirty=dirty,
+                    tagged_metadata=tagged_metadata,
+                    epoch=epoch,
+                    branch=branch,
+                    timestamp=timestamp,
+                )
+                version._matched_tag = tag
+                version._newer_unmatched_tags = unmatched
+                return version
+
         _detect_vcs(Vcs.Git)
         if tag_branch is None:
             tag_branch = "HEAD"
@@ -812,7 +943,7 @@ class Version:
             codes=[0, 128],
         )
         if code == 128:
-            return cls("0.0.0", distance=0, dirty=True, branch=branch)
+            return cls._fallback(strict, distance=0, dirty=True, branch=branch)
         commit = msg
 
         code, msg = _run_cmd('git log --no-show-signature -n 1 --pretty=format:"%cI"')
@@ -841,8 +972,8 @@ class Version:
                 distance = int(msg)
             except Exception:
                 distance = 0
-            return cls(
-                "0.0.0",
+            return cls._fallback(
+                strict,
                 distance=distance,
                 commit=commit,
                 dirty=dirty,
@@ -886,6 +1017,7 @@ class Version:
         pattern: Union[str, Pattern] = Pattern.Default,
         latest_tag: bool = False,
         full_commit: bool = False,
+        strict: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on Mercurial tags.
@@ -896,7 +1028,55 @@ class Version:
             tagged commit for a pattern match. If false, keep looking at tags
             until there is a match.
         :param full_commit: Get the full commit hash instead of the short form.
+        :param strict: When there are no tags, fail instead of falling back to 0.0.0.
         """
+        archival = _find_higher_file(".hg_archival.txt", ".hg")
+        if archival is not None:
+            content = archival.read_text("utf8")
+            data = {}
+            for line in content.splitlines():
+                parts = line.split(":", 1)
+                if len(parts) != 2:
+                    continue
+                data[parts[0].strip()] = parts[1].strip()
+
+            tag = data.get("latesttag")
+            # The distance is 1 on a new repo or on a tagged commit.
+            distance = int(data.get("latesttagdistance", 1)) - 1
+            commit = data.get("node")
+            branch = data.get("branch")
+
+            if tag is None or tag == "null":
+                return cls._fallback(strict, distance=distance, commit=commit, branch=branch)
+
+            all_tags_file = _find_higher_file(".hgtags", ".hg")
+            if all_tags_file is None:
+                all_tags = [tag]
+            else:
+                all_tags = []
+                all_tags_content = all_tags_file.read_text("utf8")
+                for line in reversed(all_tags_content.splitlines()):
+                    parts = line.split(" ", 1)
+                    if len(parts) != 2:
+                        continue
+                    all_tags.append(parts[1])
+
+            tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
+                pattern, all_tags, latest_tag
+            )
+            version = cls(
+                base,
+                stage=stage,
+                distance=distance,
+                commit=commit,
+                tagged_metadata=tagged_metadata,
+                epoch=epoch,
+                branch=branch,
+            )
+            version._matched_tag = tag
+            version._newer_unmatched_tags = unmatched
+            return version
+
         _detect_vcs(Vcs.Mercurial)
 
         code, msg = _run_cmd("hg summary")
@@ -924,8 +1104,8 @@ class Version:
                 distance = int(msg) + 1
             except Exception:
                 distance = 0
-            return cls(
-                "0.0.0",
+            return cls._fallback(
+                strict,
                 distance=distance,
                 commit=commit,
                 dirty=dirty,
@@ -958,7 +1138,10 @@ class Version:
 
     @classmethod
     def from_darcs(
-        cls, pattern: Union[str, Pattern] = Pattern.Default, latest_tag: bool = False
+        cls,
+        pattern: Union[str, Pattern] = Pattern.Default,
+        latest_tag: bool = False,
+        strict: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on Darcs tags.
@@ -968,6 +1151,7 @@ class Version:
         :param latest_tag: If true, only inspect the latest tag on the latest
             tagged commit for a pattern match. If false, keep looking at tags
             until there is a match.
+        :param strict: When there are no tags, fail instead of falling back to 0.0.0.
         """
         _detect_vcs(Vcs.Darcs)
 
@@ -990,7 +1174,9 @@ class Version:
                 distance = int(msg)
             except Exception:
                 distance = 0
-            return cls("0.0.0", distance=distance, commit=commit, dirty=dirty, timestamp=timestamp)
+            return cls._fallback(
+                strict, distance=distance, commit=commit, dirty=dirty, timestamp=timestamp
+            )
         tags = msg.splitlines()
         tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
             pattern, tags, latest_tag
@@ -1020,6 +1206,7 @@ class Version:
         pattern: Union[str, Pattern] = Pattern.Default,
         latest_tag: bool = False,
         tag_dir: str = "tags",
+        strict: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on Subversion tags.
@@ -1030,6 +1217,7 @@ class Version:
             tagged commit for a pattern match. If false, keep looking at tags
             until there is a match.
         :param tag_dir: Location of tags relative to the root.
+        :param strict: When there are no tags, fail instead of falling back to 0.0.0.
         """
         _detect_vcs(Vcs.Subversion)
 
@@ -1053,7 +1241,9 @@ class Version:
             timestamp = _parse_timestamp(msg)
 
         if not commit:
-            return cls("0.0.0", distance=0, commit=commit, dirty=dirty, timestamp=timestamp)
+            return cls._fallback(
+                strict, distance=0, commit=commit, dirty=dirty, timestamp=timestamp
+            )
         code, msg = _run_cmd('svn ls -v -r {} "{}/{}"'.format(commit, url, tag_dir))
         lines = [line.split(maxsplit=5) for line in msg.splitlines()[1:]]
         tags_to_revs = {line[-1].strip("/"): int(line[0]) for line in lines}
@@ -1062,7 +1252,9 @@ class Version:
                 distance = int(commit)
             except Exception:
                 distance = 0
-            return cls("0.0.0", distance=distance, commit=commit, dirty=dirty, timestamp=timestamp)
+            return cls._fallback(
+                strict, distance=distance, commit=commit, dirty=dirty, timestamp=timestamp
+            )
         tags_to_sources_revs = {}
         for tag, rev in tags_to_revs.items():
             code, msg = _run_cmd('svn log -v "{}/{}/{}" --stop-on-copy'.format(url, tag_dir, tag))
@@ -1096,7 +1288,10 @@ class Version:
 
     @classmethod
     def from_bazaar(
-        cls, pattern: Union[str, Pattern] = Pattern.Default, latest_tag: bool = False
+        cls,
+        pattern: Union[str, Pattern] = Pattern.Default,
+        latest_tag: bool = False,
+        strict: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on Bazaar tags.
@@ -1106,6 +1301,7 @@ class Version:
         :param latest_tag: If true, only inspect the latest tag on the latest
             tagged commit for a pattern match. If false, keep looking at tags
             until there is a match.
+        :param strict: When there are no tags, fail instead of falling back to 0.0.0.
         """
         _detect_vcs(Vcs.Bazaar)
 
@@ -1135,8 +1331,8 @@ class Version:
                 distance = int(commit) if commit is not None else 0
             except Exception:
                 distance = 0
-            return cls(
-                "0.0.0",
+            return cls._fallback(
+                strict,
                 distance=distance,
                 commit=commit,
                 dirty=dirty,
@@ -1172,7 +1368,10 @@ class Version:
 
     @classmethod
     def from_fossil(
-        cls, pattern: Union[str, Pattern] = Pattern.Default, latest_tag: bool = False
+        cls,
+        pattern: Union[str, Pattern] = Pattern.Default,
+        latest_tag: bool = False,
+        strict: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on Fossil tags.
@@ -1181,6 +1380,7 @@ class Version:
             Refer to `from_any_vcs` for more info.
         :param latest_tag: If true, only inspect the latest tag for a pattern
             match. If false, keep looking at tags until there is a match.
+        :param strict: When there are no tags, fail instead of falling back to 0.0.0.
         """
         _detect_vcs(Vcs.Fossil)
 
@@ -1206,8 +1406,8 @@ class Version:
         # The repository creation itself counts as a commit.
         total_commits = int(msg) - 1
         if total_commits <= 0:
-            return cls(
-                "0.0.0", distance=0, commit=commit, dirty=dirty, branch=branch, timestamp=timestamp
+            return cls._fallback(
+                strict, distance=0, commit=commit, dirty=dirty, branch=branch, timestamp=timestamp
             )
 
         # Based on `compute_direct_ancestors` from descendants.c in the
@@ -1241,8 +1441,8 @@ class Version:
                 distance = int(total_commits)
             except Exception:
                 distance = 0
-            return cls(
-                "0.0.0",
+            return cls._fallback(
+                strict,
                 distance=distance,
                 commit=commit,
                 dirty=dirty,
@@ -1276,7 +1476,10 @@ class Version:
 
     @classmethod
     def from_pijul(
-        cls, pattern: Union[str, Pattern] = Pattern.Default, latest_tag: bool = False
+        cls,
+        pattern: Union[str, Pattern] = Pattern.Default,
+        latest_tag: bool = False,
+        strict: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on Pijul tags.
@@ -1286,6 +1489,7 @@ class Version:
         :param latest_tag: If true, only inspect the latest tag on the latest
             tagged commit for a pattern match. If false, keep looking at tags
             until there is a match.
+        :param strict: When there are no tags, fail instead of falling back to 0.0.0.
         """
         _detect_vcs(Vcs.Pijul)
 
@@ -1302,7 +1506,7 @@ class Version:
         code, msg = _run_cmd("pijul log --limit 1 --output-format json")
         limited_commits = json.loads(msg)
         if len(limited_commits) == 0:
-            return cls("0.0.0", dirty=dirty, branch=branch)
+            return cls._fallback(strict, dirty=dirty, branch=branch)
 
         commit = limited_commits[0]["hash"]
         timestamp = _parse_timestamp(limited_commits[0]["timestamp"])
@@ -1314,8 +1518,8 @@ class Version:
         if not msg:
             # The channel creation is in the list, so offset by 1.
             distance = len(channel_commits) - 1
-            return cls(
-                "0.0.0",
+            return cls._fallback(
+                strict,
                 distance=distance,
                 commit=commit,
                 dirty=dirty,
@@ -1409,6 +1613,7 @@ class Version:
         tag_dir: str = "tags",
         tag_branch: Optional[str] = None,
         full_commit: bool = False,
+        strict: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on a detected version control system.
@@ -1434,9 +1639,14 @@ class Version:
             current branch. This is only used for Git currently.
         :param full_commit: Get the full commit hash instead of the short form.
             This is only used for Git and Mercurial.
+        :param strict: When there are no tags, fail instead of falling back to 0.0.0.
         """
-        vcs = _detect_vcs()
-        return cls._do_vcs_callback(vcs, pattern, latest_tag, tag_dir, tag_branch, full_commit)
+        vcs = _detect_vcs_from_archival()
+        if vcs is None:
+            vcs = _detect_vcs()
+        return cls._do_vcs_callback(
+            vcs, pattern, latest_tag, tag_dir, tag_branch, full_commit, strict
+        )
 
     @classmethod
     def from_vcs(
@@ -1447,6 +1657,7 @@ class Version:
         tag_dir: str = "tags",
         tag_branch: Optional[str] = None,
         full_commit: bool = False,
+        strict: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on a specific VCS setting.
@@ -1466,8 +1677,11 @@ class Version:
             current branch. This is only used for Git currently.
         :param full_commit: Get the full commit hash instead of the short form.
             This is only used for Git and Mercurial.
+        :param strict: When there are no tags, fail instead of falling back to 0.0.0.
         """
-        return cls._do_vcs_callback(vcs, pattern, latest_tag, tag_dir, tag_branch, full_commit)
+        return cls._do_vcs_callback(
+            vcs, pattern, latest_tag, tag_dir, tag_branch, full_commit, strict
+        )
 
     @classmethod
     def _do_vcs_callback(
@@ -1478,6 +1692,7 @@ class Version:
         tag_dir: str,
         tag_branch: Optional[str],
         full_commit: bool,
+        strict: bool,
     ) -> "Version":
         mapping = {
             Vcs.Any: cls.from_any_vcs,
@@ -1489,7 +1704,7 @@ class Version:
             Vcs.Fossil: cls.from_fossil,
             Vcs.Pijul: cls.from_pijul,
         }  # type: Mapping[Vcs, Callable[..., "Version"]]
-        kwargs = {"pattern": pattern, "latest_tag": latest_tag}  # type: dict
+        kwargs = {"pattern": pattern, "latest_tag": latest_tag, "strict": strict}  # type: dict
         callback = mapping[vcs]
         for kwarg, value in [
             ("tag_branch", tag_branch),
